require 'net/http'
require 'uri'
require 'http_utility/constants'

class HttpUtility
  attr_accessor :protocol, :host, :status, :body

  def initialize(protocol, host)
    @protocol = protocol || Constants::HTTP_PROTOCOL
    @host = host
  end

  def get_response(route = nil, params = {})
    uri = get_uri(route)
    uri.query = URI.encode_www_form(params)
    req = Net::HTTP::Get.new(uri)
    http_response(uri, req)
  end

  def post_response(route = '/', params = {}, headers = [])
    uri = get_uri(route)
    if headers.select do |header|
      header[:key].casecmp(Constants::CONTENT_TYPE_KEY) && header[:value].casecmp(Constants::CONTENT_TYPE_FORM_URLENCODED)
    end
      uri.query = URI.encode_www_form(params)
      req = Net::HTTP::Post.new(uri)
    elsif headers.select do |header|
      header[:key].casecmp(Constants::HEADER_CONTENT_TYPE_KEY) && header[:value].casecmp(Constants::CONTENT_TYPE_JSON)
    end
      req = Net::HTTP::Post.new(uri)
      req.body = params.to_json
    else
      req = Net::HTTP::Post.new(uri)
    end
    headers.each { |header| req[header[:key]] = header[:value] }
    http_response(uri, req)
  end

  def get_uri(route)
    route ? URI("#{@protocol}#{@host}/#{route}") : URI("#{@protocol}#{@host}")
  end

  def http_response(uri, req)
    return unless uri.hostname && uri.port
    Net::HTTP.start(uri.hostname, uri.port) { |http| http.request(req) }
  end
end